@@ -1,11 +1,11 @@
 """Two-player nonlocal game."""
-from typing import Dict, Tuple
+from typing import Dict, List, Tuple, Union
 from collections import defaultdict
 import cvxpy
 import numpy as np
 from toqito.random import random_povm
 from toqito.matrix_ops import tensor
-from toqito.helper import update_odometer
+from toqito.helper import update_odometer, npa_constraints
 
 
 class NonlocalGame:
@@ -69,6 +69,67 @@ class NonlocalGame:
             self.pred_mat = pred_mat2
             self.reps = reps
 
+    @classmethod
+    def from_bcs_game(cls, constraints: List[np.ndarray], reps: int = 1) -> "NonlocalGame":
+        """
+        Construct nonlocal game object from a binary constraint system game.
+
+        :param constraints: A list of m matrices corresponding to the `m`
+                            constraints in the BCS game. Each constraint matrix is
+                            an `n`-dimensional matrix of shape `2 x 2 x ... x 2`.
+                            The `(i, j, k, ...)`-th entry is 1 if and only if the
+                            contraint is satisfied given the values v_1 = i, v_2 = j,
+                            v_3 = k, ..., and otherwise 0.
+        :param reps: Number of parallel repetitions to perform. Default is 1.
+
+        :return: An instance of a nonlocal game object.
+        """
+        num_constraints = len(constraints)
+        if num_constraints == 0:
+            raise ValueError("At least 1 constraint is required")
+        num_variables = constraints[0].ndim
+
+        # Retrieve dependent variables for each constraint
+        dependent_variables = np.zeros((num_constraints, num_variables))
+
+        # `v_i` is _not_ a dependent variable of `c_j` if all entries in
+        # the `i`-th dimension of `constraints[j]` are equal, i.e.:
+        #   c_j[:, ..., :, 0, : ..., :] == c_j[:, ..., :, 1, : ..., :]
+        for j in range(num_constraints):
+            for i in range(num_variables):
+                dependent_variables[j, i] = np.diff(constraints[j], axis=i).any()
+
+        # Calculate probability matrix P(x, y) where:
+        #   x: uniformly randomly-selected constraint `c_x`
+        #   y: uniformly randomly-selected variable `v_y` in `c_x`
+        prob_mat = np.zeros((num_constraints, num_variables))
+        for j in range(num_constraints):
+            p_x = 1.0 / num_constraints
+            num_dependent_vars = dependent_variables[j].sum()
+            p_y = dependent_variables[j] / num_dependent_vars
+            prob_mat[j] = p_x * p_y
+
+        # Compute prediction matrix of outcomes given questions and answer pairs:
+        #   a: Alice's truth assignment to all variables in `c_x`
+        #   b: Bob's truth assignment for `v_y` in `c_x`
+        pred_mat = np.zeros((2 ** num_variables, 2, num_constraints, num_variables))
+        for x_ques in range(num_constraints):
+            for a_ans in range(pred_mat.shape[0]):
+                # Convert to binary representation
+                bin_a = [int(x) for x in np.binary_repr(a_ans)]
+                truth_assignment = np.zeros(num_variables, dtype=np.int8)
+                truth_assignment[-len(bin_a) :] = bin_a
+                truth_assignment = tuple(truth_assignment)
+
+                for y_ques in range(num_variables):
+                    # The verifier can only accept the answer if Bob's truth assignment
+                    # is consistent with Alice's
+                    b_ans = truth_assignment[y_ques]
+
+                    pred_mat[a_ans, b_ans, x_ques, y_ques] = constraints[x_ques][truth_assignment]
+
+        return cls(prob_mat, pred_mat, reps)
+
     def classical_value(self) -> float:
         """
         Compute the classical value of the nonlocal game.
@@ -125,6 +186,7 @@ class NonlocalGame:
 
     def quantum_value_lower_bound(
         self,
+        dim: int = 2,
         iters: int = 5,
         tol: float = 10e-6,
     ):
@@ -133,7 +195,7 @@ class NonlocalGame:
 
         Calculates a lower bound on the maximum value that the specified
         nonlocal game can take on in quantum mechanical settings where Alice and
-        Bob each have access to `d`-dimensional quantum system.
+        Bob each have access to `dim`-dimensional quantum system.
 
         This function works by starting with a randomly-generated POVM for Bob,
         and then optimizing Alice's POVM and the shared entangled state. Then
@@ -257,6 +319,8 @@ class NonlocalGame:
             Physical Review A 75.4 (2007): 042103.
             https://arxiv.org/abs/quant-ph/0608128
 
+        :param dim: The dimension of the quantum system that Alice and Bob have
+                    access to (default = 2).
         :param iters: The number of times to run the alternating projection
                       algorithm.
         :param tol: The tolerance before quitting out of the alternating
@@ -271,7 +335,7 @@ class NonlocalGame:
             # Generate a set of random POVMs for Bob. These measurements serve
             # as a rough starting point for the alternating projection
             # algorithm.
-            bob_tmp = random_povm(num_outputs_bob, num_inputs_bob, num_outputs_bob)
+            bob_tmp = random_povm(dim, num_inputs_bob, num_outputs_bob)
             bob_povms = defaultdict(int)
             for y_ques in range(num_inputs_bob):
                 for b_ans in range(num_outputs_bob):
@@ -286,23 +350,21 @@ class NonlocalGame:
                 # Bob's. If this is the first iteration, then the previously
                 # randomly generated operators in the outer loop are Bob's.
                 # Otherwise, Bob's operators come from running the next SDP.
-                alice_povms, lower_bound = self.__optimize_alice(bob_povms)
-                bob_povms, lower_bound = self.__optimize_bob(alice_povms)
+                alice_povms, lower_bound = self.__optimize_alice(dim, bob_povms)
+                bob_povms, lower_bound = self.__optimize_bob(dim, alice_povms)
 
                 it_diff = lower_bound - prev_win
                 prev_win = lower_bound
 
                 # As the SDPs keep alternating, check if the winning probability
                 # becomes any higher. If so, replace with new best.
-                if best < lower_bound:
-                    best = lower_bound
+                best = max(best, lower_bound)
 
-            if best_lower_bound < best:
-                best_lower_bound = best
+            best_lower_bound = max(best, best_lower_bound)
 
         return best_lower_bound
 
-    def __optimize_alice(self, bob_povms) -> Tuple[Dict, float]:
+    def __optimize_alice(self, dim, bob_povms) -> Tuple[Dict, float]:
         """Fix Bob's measurements and optimize over Alice's measurements."""
         # Get number of inputs and outputs.
         (
@@ -319,11 +381,9 @@ class NonlocalGame:
         alice_povms = defaultdict(cvxpy.Variable)
         for x_ques in range(num_inputs_alice):
             for a_ans in range(num_outputs_alice):
-                alice_povms[x_ques, a_ans] = cvxpy.Variable(
-                    (num_outputs_alice, num_inputs_alice), PSD=True
-                )
+                alice_povms[x_ques, a_ans] = cvxpy.Variable((dim, dim), PSD=True)
 
-        tau = cvxpy.Variable((num_outputs_alice, num_outputs_bob), PSD=True)
+        tau = cvxpy.Variable((dim, dim), PSD=True)
 
         # .. math::
         #    \sum_{(x,y) \in \Sigma} \pi(x, y) V(a,b|x,y) \ip{B_b^y}{A_a^x}
@@ -376,7 +436,7 @@ class NonlocalGame:
         lower_bound = problem.solve()
         return alice_povms, lower_bound
 
-    def __optimize_bob(self, alice_povms) -> Tuple[Dict, float]:
+    def __optimize_bob(self, dim, alice_povms) -> Tuple[Dict, float]:
         """Fix Alice's measurements and optimize over Bob's measurements."""
         # Get number of inputs and outputs.
         (
@@ -391,9 +451,7 @@ class NonlocalGame:
         bob_povms = defaultdict(cvxpy.Variable)
         for y_ques in range(num_inputs_bob):
             for b_ans in range(num_outputs_bob):
-                bob_povms[y_ques, b_ans] = cvxpy.Variable(
-                    (num_outputs_bob, num_outputs_bob), PSD=True
-                )
+                bob_povms[y_ques, b_ans] = cvxpy.Variable((dim, dim), PSD=True)
 
         win = 0
         for x_ques in range(num_inputs_alice):
@@ -416,7 +474,7 @@ class NonlocalGame:
             bob_sum_b = 0
             for b_ans in range(num_outputs_bob):
                 bob_sum_b += bob_povms[y_ques, b_ans]
-            constraints.append(bob_sum_b == np.identity(num_outputs_bob))
+            constraints.append(bob_sum_b == np.identity(dim))
 
         problem = cvxpy.Problem(objective, constraints)
 
@@ -519,3 +577,54 @@ class NonlocalGame:
         ns_val = problem.solve()
 
         return ns_val
+
+    def commuting_measurement_value_upper_bound(self, k: Union[int, str] = 1) -> float:
+        """
+        Compute an upper bound on the commuting measurement value of the nonlocal game.
+
+        This function calculates an upper bound on the commuting measurement value by
+        using k-levels of the NPA hierarchy [NPA]_. The NPA hierarchy is a uniform family
+        of semidefinite programs that converges to the commuting measurement value of
+        any nonlocal game.
+
+        You can determine the level of the hierarchy by a positive integer or a string
+        of a form like '1+ab+aab', which indicates that an intermediate level of the hierarchy
+        should be used, where this example uses all products of one measurement, all products of
+        one Alice and one Bob measurement, and all products of two Alice and one Bob measurements.
+
+        References
+        ==========
+        .. [NPA] Miguel Navascues, Stefano Pironio, Antonio Acin,
+            "A convergent hierarchy of semidefinite programs characterizing the
+            set of quantum correlations."
+            https://arxiv.org/abs/0803.4290
+
+        :param k: The level of the NPA hierarchy to use (default=1).
+        :return: The upper bound on the commuting strategy value of a nonlocal game.
+        """
+        alice_out, bob_out, alice_in, bob_in = self.pred_mat.shape
+
+        mat = defaultdict(cvxpy.Variable)
+        for x_in in range(alice_in):
+            for y_in in range(bob_in):
+                mat[x_in, y_in] = cvxpy.Variable(
+                    (alice_out, bob_out), name="M(a, b | {}, {})".format(x_in, y_in)
+                )
+
+        p_win = cvxpy.Constant(0)
+        for a_out in range(alice_out):
+            for b_out in range(bob_out):
+                for x_in in range(alice_in):
+                    for y_in in range(bob_in):
+                        p_win += (
+                            self.prob_mat[x_in, y_in]
+                            * self.pred_mat[a_out, b_out, x_in, y_in]
+                            * mat[x_in, y_in][a_out, b_out]
+                        )
+
+        npa = npa_constraints(mat, k)
+        objective = cvxpy.Maximize(p_win)
+        problem = cvxpy.Problem(objective, npa)
+        cs_val = problem.solve()
+
+        return cs_val
